﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;

using Esri.ArcGISRuntime.Controls;
using Esri.ArcGISRuntime.Layers;

namespace MapViewRepro
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        public MainWindow()
        {
            MapViewModel = new MapViewModel();
            DataContext = this;
            InitializeComponent();
        }

        public MapViewModel MapViewModel { get; private set; }
    }


    public class MapViewModel
    {
        
        public MapViewModel()
        {
            var _ = Initialize();
        }

        private bool _isInitialized;
        /// <summary>
        /// Gets a value indicating whether ViewModel is initialized.
        /// </summary>
        public bool IsInitialized
        {
            get { return _isInitialized; }
            private set
            {
                _isInitialized = value;
            }
        }

        /// <summary>
        /// Initialized ViewModel.
        /// Override to implement custom initialization logic for ViewModel.
        /// </summary>
        public async Task Initialize()
        {
            await InitializeAsync();
            IsInitialized = true;
        }

        private async Task InitializeAsync()
        {
            var url = "http://services.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer";
            // Basemap layer from ArcGIS Online hosted service
            var basemap = new ArcGISTiledMapServiceLayer
            {
                ID = "Basemap",
                DisplayName = "Basemap",
                ServiceUri = url
            };

            // Initialize layer in Try - Catch 
            Exception exceptionToHandle = null;
            try
            {
                await basemap.InitializeAsync();
                Map.Layers.Add(basemap);
                Map.InitialViewpoint = new Viewpoint(basemap.FullExtent);
                Map.SpatialReference = basemap.DefaultSpatialReference;
            }
            catch (Exception exception)
            {
                // Exception is thrown ie if layer url is not found - ie. {"Error code '400' : 'Invalid URL'"} 
                exceptionToHandle = exception;
            }

            if (exceptionToHandle != null)
            {
                // Initialization failed, show message and return
                //await MessageService.Instance.ShowMessage(string.Format(
                Trace.WriteLine(String.Format(
                    "Could not create basemap. Error = {0}", exceptionToHandle),
                    "An error occured");
            }
        }
        private Esri.ArcGISRuntime.Controls.Map _map;
        public Esri.ArcGISRuntime.Controls.Map Map
        {
            get
            {
                return _map ?? (_map = new Esri.ArcGISRuntime.Controls.Map());
            }
        }
    }
}
